package org.xbib.event.clock;

import java.time.Instant;
import java.util.concurrent.Callable;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.event.bus.EventBus;

public class ClockEventService implements Callable<Integer> {

    private static final Logger logger = Logger.getLogger(ClockEventService.class.getName());

    private final EventBus eventBus;

    private final Class<? extends ClockEvent> eventClass;

    public ClockEventService(EventBus eventBus,
                             Class<? extends ClockEvent> eventClass) {
        this.eventBus = eventBus;
        this.eventClass = eventClass;
    }

    @Override
    public Integer call() throws Exception {
        try {
            logger.log(Level.FINE, "posting clock event " + eventClass.getName());
            ClockEvent clockEvent = eventClass.getDeclaredConstructor().newInstance();
            clockEvent.setInstant(Instant.now());
            eventBus.post(clockEvent);
            return 0;
        } catch (Throwable t) {
            logger.log(Level.WARNING, t.getMessage(), t);
            return 1;
        }
    }
}
