package org.xbib.event.syslog;

import org.xbib.event.bus.AsyncEventBus;
import org.xbib.event.io.file.FileFollowEventService;
import org.xbib.settings.Settings;

import java.io.Closeable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

public class SyslogEventManager implements Closeable {
    private static final Logger logger = Logger.getLogger(SyslogEventManager.class.getName());

    private final List<SyslogService> syslogServices;

    public SyslogEventManager(Settings settings,
                              AsyncEventBus eventBus) {
        this.syslogServices = new ArrayList<>();
        for (Map.Entry<String, Settings> entry : settings.getGroups("event.syslog").entrySet()) {
            Settings definition = entry.getValue();
            if (definition.getAsBoolean("enabled", true)) {
                try {
                    SyslogService syslogService = new SyslogService(definition, eventBus);
                    syslogServices.add(syslogService);
                    logger.log(Level.INFO, "syslog service " + entry.getKey() + " added");
                } catch (Exception e) {
                    logger.log(Level.SEVERE, "unable to create file follow service " + entry.getKey() + ", reason " + e.getMessage(), e);
                }
            }
        }
    }

    @Override
    public void close() throws IOException {
        for (SyslogService syslogService : syslogServices) {
            syslogService.close();
        }
    }
}
