package org.xbib.event.yield.ops;

import org.xbib.event.yield.Advancer;
import org.xbib.event.yield.Query;
import org.xbib.event.yield.Traverser;
import org.xbib.event.yield.Yield;

import java.util.function.Predicate;

public class TakeWhile<T> implements Advancer<T>, Traverser<T> {

    private final Query<T> upstream;

    private final Predicate<? super T> predicate;

    private boolean hasNext;

    public TakeWhile(Query<T> upstream, Predicate<? super T> predicate) {
        this.upstream = upstream;
        this.predicate = predicate;
        this.hasNext = true;
    }

    @Override
    public boolean tryAdvance(Yield<? super T> yield) {
        if (!hasNext) return false; // Once predicate is false it finishes the iteration
        Yield<T> takeWhile = item -> {
            if (predicate.test(item)) {
                yield.ret(item);
            } else {
                hasNext = false;
            }
        };
        return upstream.tryAdvance(takeWhile) && hasNext;
    }

    @Override
    public void traverse(Yield<? super T> yield) {
        upstream.shortCircuit(item -> {
            if (!predicate.test(item)) Yield.bye();
            yield.ret(item);
        });
    }
}
